"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const child_process_1 = require("child_process");
const path_1 = require("path");
const symbol_sdk_1 = require("symbol-sdk");
const symbol_openapi_typescript_fetch_client_1 = require("symbol-openapi-typescript-fetch-client");
const sleep_1 = require("./sleep");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class RunService {
    constructor(params) {
        this.params = params;
        this.promisePoll = (promiseFunction, totalPollingTime, pollIntervalMs) => {
            const startTime = new Date().getMilliseconds();
            return promiseFunction().then(async (result) => {
                if (result) {
                    return true;
                }
                else {
                    const endTime = new Date().getMilliseconds();
                    const newPollingTime = Math.max(totalPollingTime - pollIntervalMs - (endTime - startTime), 0);
                    if (newPollingTime) {
                        await sleep_1.sleep(pollIntervalMs);
                        return this.promisePoll(promiseFunction, newPollingTime, pollIntervalMs);
                    }
                    else {
                        return false;
                    }
                }
            });
        };
    }
    async run() {
        const basicArgs = ['up', '--remove-orphans'];
        if (this.params.daemon) {
            basicArgs.push('--detach');
        }
        if (this.params.build) {
            basicArgs.push('--build');
        }
        await this.basicRun(basicArgs);
        if (this.params.daemon) {
            await this.pollServiceUntilIsUp();
        }
    }
    async pollServiceUntilIsUp(totalPollingTime = 30000, pollIntervalMs = 2000) {
        const url = 'http://localhost:3000';
        const repositoryFactory = new symbol_sdk_1.RepositoryFactoryHttp(url);
        const nodeRepository = repositoryFactory.createNodeRepository();
        const pollFunction = async () => {
            logger.info(`Testing ${url}/node/health`);
            try {
                const healthStatus = await nodeRepository.getNodeHealth().toPromise();
                if (healthStatus.apiNode === symbol_openapi_typescript_fetch_client_1.NodeStatusEnum.Down) {
                    logger.warn(`Network is NOT up and running YET: Api Node is still Down!`);
                    return false;
                }
                if (healthStatus.db === symbol_openapi_typescript_fetch_client_1.NodeStatusEnum.Down) {
                    logger.warn(`Network is NOT up and running YET: DB is still Down!`);
                    return false;
                }
                logger.info(`Network IS up and running...`);
                return true;
            }
            catch (e) {
                logger.warn(`Network is NOT up and running YET: ${e.message}`);
                return false;
            }
        };
        const started = await this.promisePoll(pollFunction, totalPollingTime, pollIntervalMs);
        if (!started) {
            throw new Error(`Network did NOT start!!!`);
        }
    }
    async stop() {
        await this.basicRun(['down']);
    }
    async basicRun(extraArgs) {
        const target = `${this.params.target}`;
        const dockerFile = `${target}${path_1.sep}docker${path_1.sep}docker-compose.yml`;
        const dockerComposeArgs = ['-f', dockerFile];
        const cmd = child_process_1.spawn('docker-compose', [...dockerComposeArgs, ...extraArgs]);
        const log = (data) => {
            if (!this.params.daemon)
                console.log(`${data}`.trim());
        };
        cmd.stdout.on('data', (data) => {
            log(data);
        });
        cmd.stderr.on('data', (data) => {
            log(`${data}`.trim());
        });
        cmd.on('error', (error) => {
            log(`error: ${error.message}`.trim());
        });
        cmd.on('exit', (code, signal) => {
            log(`child process exited with code ${code} and signal ${signal}`);
        });
        cmd.on('close', (code) => {
            log(`child process exited with code ${code}`);
        });
        process.on('SIGINT', function () {
            log('Received SIGINT signal');
        });
        logger.info(`running: docker-compose ${dockerComposeArgs.join(' ')}`);
    }
}
exports.RunService = RunService;
